@@ -1,17 +1,13 @@
-from abc import ABCMeta, abstractmethod
-from itertools import chain
-from collections import OrderedDict
-from mewpy.utils.constants import EAConstants
-from mewpy.simulation import get_simulator, SimulationMethod, SStatus
-from mewpy.utils.ode import ODEStatus
-import numpy as np
-from functools import reduce
 import math
 import warnings
+from abc import ABCMeta, abstractmethod
+from functools import reduce
+import numpy as np
+from ..simulation import get_simulator, SimulationMethod, SStatus
+from ..util.constants import EAConstants
 
 
 class EvaluationFunction:
-   
     __metaclass__ = ABCMeta
 
     def __init__(self, maximize=True, worst_fitness=0.0):
@@ -23,18 +19,17 @@ class EvaluationFunction:
     @abstractmethod
     def get_fitness(self, simul_results, candidate, **kwargs):
         """Evaluates a candidate
-        
+
         :param simul_results: (dic) A dictionary of phenotype SimulationResult objects
         :param candidate:  Candidate beeing evaluated
         :returns: A fitness value.
 
         """
-        
-        return
+        raise NotImplementedError
 
     @abstractmethod
     def method_str(self):
-        return
+        raise NotImplementedError
 
     def short_str(self):
         return self.method_str
@@ -64,33 +59,35 @@ class EvaluationFunction:
 
 
 class PhenotypeEvaluationFunction(EvaluationFunction):
-    
+
     def __init__(self, maximize=True, worst_fitness=0.0):
-        super(PhenotypeEvaluationFunction, self).__init__(maximize=maximize, worst_fitness=0.0)
+        super(PhenotypeEvaluationFunction, self).__init__(maximize=maximize, worst_fitness=worst_fitness)
 
 
 class KineticEvaluationFunction(EvaluationFunction):
-    
+
     def __init__(self, maximize=True, worst_fitness=0.0):
-        super(KineticEvaluationFunction, self).__init__(maximize=maximize, worst_fitness=0.0)
+        super(KineticEvaluationFunction, self).__init__(maximize=maximize, worst_fitness=worst_fitness)
 
 
 class TargetFlux(PhenotypeEvaluationFunction):
-    """ Target Flux evaluation function. 
+    """ Target Flux evaluation function.
     The fitness value is the flux value of the identified reaction.
     If the reaction parameter is None, the fitness value is the optimization objective value.
-    Additional parameters include a minimum of allowed biomass value computed from the min_biomass_per and reference flux values
+    Additional parameters include a minimum of allowed biomass value computed from the min_biomass_per
+    and reference flux values
 
-    
-    :param reaction: (str) The reaction identifier whose flux value is to be used as fitness. Default None in which case the model objective is considered.
+    :param reaction: (str) The reaction identifier whose flux value is to be used as fitness. Default None \
+        in which case the model objective is considered.
     :param biomass: (str) The biomass reaction identifier.
     :param maximize: (boolean) The optimization direction. Default True for maximization.
     :param min_biomass_value: (float) The minimum biomass value.
     :param min_biomass_per: (float) Minimum biomass percentage. Only used if no min_biomass_value is provided.
-    
+
     """
 
-    def __init__(self, reaction, biomass=None, maximize=True, min_biomass_value=None, min_biomass_per=0.0, method=SimulationMethod.pFBA):
+    def __init__(self, reaction, biomass=None, maximize=True, min_biomass_value=None, min_biomass_per=0.0,
+                 method=SimulationMethod.pFBA):
         super(TargetFlux, self).__init__(maximize=maximize, worst_fitness=0.0)
         self.reaction = reaction
         self.biomass = biomass
@@ -100,7 +97,7 @@ class TargetFlux(PhenotypeEvaluationFunction):
 
     def get_fitness(self, simul_results, candidate, **kwargs):
         """Evaluates a candidate
-        
+
         :param simul_results: (dic) A dictionary of phenotype SimulationResult objects
         :param candidate:  Candidate beeing evaluated
         :returns: A fitness value.
@@ -140,22 +137,26 @@ class TargetFlux(PhenotypeEvaluationFunction):
         return "TargetFlux"
 
     def method_str(self):
-        return "TargetFlux {} with at least {} of biomass ({})".format(self.reaction, self.min_biomass_per, self.biomass)
+        return "TargetFlux {} with at least {} of biomass ({})".format(self.reaction, self.min_biomass_per,
+                                                                       self.biomass)
 
 
-class WYIELD (PhenotypeEvaluationFunction):
-    """ Weighted Yield (WYIELD) objective function, a linear combination of the target product minimum and maximum FVA under the introduced metabolic modifications.
+class WYIELD(PhenotypeEvaluationFunction):
+    """ Weighted Yield (WYIELD) objective function, a linear combination of the target product minimum and
+    maximum FVA under the introduced metabolic modifications.
 
     :param biomassId: (str) Biomass reaction identifier.
     :param productId: (str) Target product reaction identifier.
-    
+
     kwargs options:
-    
+
     :param min_biomass_value: (float) Minimum biomass value (default None, in which case the min_biomass_per is used).
-    :param min_biomass_per: (float) Instead of defining explicitly a minimum biomass value, a percentage of the wild type biomass is used. Only used when no min_biomass_value is defined (default min_biomass_per: 0.10).                          
-    :param alpha: (float) Tradeoff between the Max and min FVA of the target product (alpha Max + (1-alpha) min). Must be in range [0,1]  (default alpha: 0.3).
-    :param scale: (boolean) Defines if the WYIELD is devided by the biomass of the simulated result (default false).  
-    
+    :param min_biomass_per: (float) Instead of defining explicitly a minimum biomass value, a percentage of the wild \
+        type biomass is used. Only used when no min_biomass_value is defined (default min_biomass_per: 0.10).
+    :param alpha: (float) Tradeoff between the Max and min FVA of the target product (alpha Max + (1-alpha) min). \
+        Must be in range [0,1]  (default alpha: 0.3).
+    :param scale: (boolean) Defines if the WYIELD is devided by the biomass of the simulated result (default false).
+
     """
 
     def __init__(self, biomassId, productId, maximize=True, **kwargs):
@@ -175,7 +176,7 @@ class WYIELD (PhenotypeEvaluationFunction):
 
     def get_fitness(self, simul_results, candidate, **kwargs):
         """Evaluates a candidate
-        
+
         :param simul_results: (dic) A dictionary of phenotype SimulationResult objects
         :param candidate:  Candidate beeing evaluated
         :returns: A fitness value.
@@ -202,7 +203,7 @@ class WYIELD (PhenotypeEvaluationFunction):
         ids = list(ssFluxes.keys())
         if self.biomassId not in ids or self.productId not in ids:
             raise ValueError(
-                "Reaction ids are not present in the fluxes distribution. Please check if the objective function ids are correct.")
+                "Reaction ids are not present in the fluxes distribution.")
 
         biomassFluxValue = ssFluxes[self.biomassId] * 0.999
 
@@ -210,7 +211,7 @@ class WYIELD (PhenotypeEvaluationFunction):
             # computed only once
             if self.min_biomass_value is None or self.min_biomass_value < 0.0:
                 solution = simulation.simulate(
-                        objective={self.biomassId: 1}, scalefactor=scalefactor)
+                    objective={self.biomassId: 1}, scalefactor=scalefactor)
                 wtBiomassValue = solution.fluxes[self.biomassId]
                 minBiomass = wtBiomassValue * self.min_biomass_per
                 self.min_biomass_value = minBiomass
@@ -223,7 +224,7 @@ class WYIELD (PhenotypeEvaluationFunction):
             constraints[self.biomassId] = (biomassFluxValue, 100000.0)
 
             # only need to simulate FVA max if alpha is larger than 0, otherwise it will always be zero
-            if(self.alpha > 0):
+            if (self.alpha > 0):
                 fvaMaxResult = simulation.simulate(
                     objective={self.productId: 1}, constraints=constraints, scalefactor=scalefactor)
                 if fvaMaxResult.status == SStatus.OPTIMAL:
@@ -232,9 +233,9 @@ class WYIELD (PhenotypeEvaluationFunction):
                     return self.no_solution
 
             # only need to simulate FVA min if alpha is lesser than 1, otherwise it will always be zero
-            if(self.alpha < 1):
+            if (self.alpha < 1):
                 fvaMinResult = simulation.simulate(objective={
-                                                   self.productId: 1}, constraints=constraints, maximize=False, scalefactor=scalefactor)
+                    self.productId: 1}, constraints=constraints, maximize=False, scalefactor=scalefactor)
                 if fvaMinResult.status == SStatus.OPTIMAL:
                     fvaMinProd = fvaMinResult.fluxes[self.productId]
                 else:
@@ -248,7 +249,7 @@ class WYIELD (PhenotypeEvaluationFunction):
                 if self.scale:
                     res = res / biomassFluxValue
             return res
-        except:
+        except Exception:
             return self.no_solution
 
     def required_simulations(self):
@@ -266,19 +267,21 @@ class BPCY(PhenotypeEvaluationFunction):
     This class implements the "Biomass-Product Coupled Yield" objective function. The fitness is given by the equation:
     (biomass_flux * product_flux)/ uptake_flux
 
-    
     :param biomass: (str) Biomass reaction identifier
     :param product: (str) Target product reaction identifier
-    :param uptake: (str) (optional) Reaction of uptake. If no substract is defined, ie uptake is None, a substract flux value of 1.0 is considered.  
+    :param uptake: (str) (optional) Reaction of uptake. If no substract is defined, ie uptake is None, a substract \
+        flux value of 1.0 is considered.
 
     kargs options:
-    
-    :param method: (SimulationMethod) The simulation method. Default Node in which case received simulation results are used to compute the biomass product coupled yield.
-    :param reference: (dic) Wild type reference values when MOMA, lMOMA or ROOM are defined as method. If not provided, wild type reference values will be computed.
-    
+
+    :param method: (SimulationMethod) The simulation method. Default Node in which case received simulation results \
+        are used to compute the biomass product coupled yield.
+    :param reference: (dic) Wild type reference values when MOMA, lMOMA or ROOM are defined as method. \
+        If not provided, wild type reference values will be computed.
+
     """
 
-    def __init__(self, biomass, product, uptake=None,  maximize=True, **kwargs):
+    def __init__(self, biomass, product, uptake=None, maximize=True, **kwargs):
         super(BPCY, self).__init__(maximize=maximize, worst_fitness=0.0)
         self.biomassId = biomass
         self.productId = product
@@ -289,7 +292,7 @@ class BPCY(PhenotypeEvaluationFunction):
 
     def get_fitness(self, simul_results, candidate, **kwargs):
         """Evaluates a candidate
-        
+
         :param simul_results: (dic) A dictionary of phenotype SimulationResult objects
         :param candidate:  Candidate beeing evaluated
         :returns: A fitness value.
@@ -318,7 +321,7 @@ class BPCY(PhenotypeEvaluationFunction):
         if EAConstants.DEBUG:
             try:
                 print("BPCY Bionamss: {} product: {}".format(ssFluxes[self.biomassId], ssFluxes[self.productId]))
-            except:
+            except Exception:
                 print("BPCY No Fluxes")
         return (ssFluxes[self.biomassId] * ssFluxes[self.productId]) / uptake
 
@@ -335,26 +338,31 @@ class BPCY(PhenotypeEvaluationFunction):
             return "BPCY (" + self.biomassId + " * " + self.productId + ")"
 
 
-class BPCY_FVA (PhenotypeEvaluationFunction):
+class BPCY_FVA(PhenotypeEvaluationFunction):
     """
-    This class implements the "Biomass-Product Coupled Yield" objective function with FVA as defined in 
-    "OptRAM: In-silico strain design via integrative regulatory-metabolic network modeling". It combines BPCY with WYIELD objective functions. 
+    This class implements the "Biomass-Product Coupled Yield" objective function with FVA as defined in
+    "OptRAM: In-silico strain design via integrative regulatory-metabolic network modeling".
+    It combines BPCY with WYIELD objective functions.
 
     The fitness is given by the equation:
-    ((biomass_flux * product_flux) * / uptake_flux ) * (1-log((range)/(target))) where range=(FVA_max-FVA_min)/2 and target= (FVA_max+FVA_min)/2
-    
+    ((biomass_flux * product_flux) * / uptake_flux ) * (1-log((range)/(target))) where range=(FVA_max-FVA_min)/2
+    and target= (FVA_max+FVA_min)/2
+
     :param biomass: (str) Biomass reaction identifier.
     :param product: (str) Target product reaction identifier.
-    :param uptake: (str) (optional) Reaction of uptake. If no substract is defined, ie uptakeId is None, a substract flux value of 1.0 is considered.  
+    :param uptake: (str) (optional) Reaction of uptake. If no substract is defined, ie uptakeId is None, a substract \
+        flux value of 1.0 is considered.
 
     kwargs options:
-    
-    :param method: (SimulationMethod) The simulation method. Default Node in which case received simulation results are used to compute the biomass product coupled yield.
-    :param reference: (dic) Wild type reference values when MOMA, lMOMA or ROOM are defined as method. If not provided, wild type reference values will be computed.
-    
+
+    :param method: (SimulationMethod) The simulation method. Default Node in which case received simulation results \
+        are used to compute the biomass product coupled yield.
+    :param reference: (dic) Wild type reference values when MOMA, lMOMA or ROOM are defined as method. \
+        If not provided, wild type reference values will be computed.
+
     """
 
-    def __init__(self, biomass, product, uptake=None,  maximize=True, **kwargs):
+    def __init__(self, biomass, product, uptake=None, maximize=True, **kwargs):
         super(BPCY_FVA, self).__init__(maximize=maximize, worst_fitness=0.0)
         self.biomassId = biomass
         self.productId = product
@@ -365,7 +373,7 @@ class BPCY_FVA (PhenotypeEvaluationFunction):
 
     def get_fitness(self, simul_results, candidate, **kwargs):
         """Evaluates a candidate.
-        
+
         :param simul_results: (dic) A dictionary of phenotype SimulationResult objects
         :param candidate:  Candidate beeing evaluated
         :returns: A fitness value.
@@ -415,7 +423,8 @@ class BPCY_FVA (PhenotypeEvaluationFunction):
         if abs(v_max) == abs(v_min):
             return (ssFluxes[self.biomassId] * ssFluxes[self.productId]) / uptake
         else:
-            return ((ssFluxes[self.biomassId] * ssFluxes[self.productId]) / uptake) * (1-math.log(abs((v_max-v_min)/(v_max+v_min))))
+            return ((ssFluxes[self.biomassId] * ssFluxes[self.productId]) / uptake) * (
+                1 - math.log(abs((v_max - v_min) / (v_max + v_min))))
 
     def required_simulations(self):
         return [self.method]
@@ -430,13 +439,13 @@ class BPCY_FVA (PhenotypeEvaluationFunction):
             return "BPCY_FVA (" + self.biomassId + " * " + self.productId + ")"
 
 
-class AggregatedSum(PhenotypeEvaluationFunction,KineticEvaluationFunction):
+class AggregatedSum(PhenotypeEvaluationFunction, KineticEvaluationFunction):
     """
-    Aggredated sum evaluation function. Used to converte MOEAs into Single Objective EAs. 
+    Aggredated sum evaluation function. Used to converte MOEAs into Single Objective EAs.
 
-        
     :param fevaluation: (list) List of evaluation functions.
-    :param tradeoffs: (list) Tradeoff values for each evaluation function. If None, all functions have the same associated weight.
+    :param tradeoffs: (list) Tradeoff values for each evaluation function. If None, all functions have \
+        the same associated weight.
 
     """
 
@@ -447,7 +456,7 @@ class AggregatedSum(PhenotypeEvaluationFunction,KineticEvaluationFunction):
         if tradeoffs and len(tradeoffs) == len(fevaluation):
             self.tradeoffs = tradeoffs
         else:
-            self.tradeoffs = [1/len(self.fevaluation)] * \
+            self.tradeoffs = [1 / len(self.fevaluation)] * \
                 (len(self.fevaluation))
 
     def required_simulations(self):
@@ -458,7 +467,7 @@ class AggregatedSum(PhenotypeEvaluationFunction,KineticEvaluationFunction):
 
     def get_fitness(self, simul_results, candidate, **kwargs):
         """Evaluates a candidate
-        
+
         :param simul_results: (dic) A dictionary of phenotype SimulationResult objects
         :param candidate:  Candidate beeing evaluated
         :returns: A fitness value.
@@ -474,7 +483,7 @@ class AggregatedSum(PhenotypeEvaluationFunction,KineticEvaluationFunction):
         return "Agg"
 
     def method_str(self):
-        return "Aggregated Sum = " + reduce(lambda a, b: a+" "+b, [f.method_str() for f in self.fevaluation], "")
+        return "Aggregated Sum = " + reduce(lambda a, b: a + " " + b, [f.method_str() for f in self.fevaluation], "")
 
 
 class MinCandSize(PhenotypeEvaluationFunction, KineticEvaluationFunction):
@@ -484,7 +493,7 @@ class MinCandSize(PhenotypeEvaluationFunction, KineticEvaluationFunction):
     during optimization.
 
     :param maxCandidateSize: (int) Maximum size allowed for candidate.
-    
+
     """
 
     def __init__(self, candidate_max_size=EAConstants.MAX_SOLUTION_SIZE, maximize=True):
@@ -492,7 +501,7 @@ class MinCandSize(PhenotypeEvaluationFunction, KineticEvaluationFunction):
         self.candidate_max_size = candidate_max_size
 
     def get_fitness(self, simulResult, candidate, **kwargs):
-        return 1 - len(candidate)/self.candidate_max_size
+        return 1 - len(candidate) / self.candidate_max_size
 
     def required_simulations(self):
         """
@@ -507,27 +516,25 @@ class MinCandSize(PhenotypeEvaluationFunction, KineticEvaluationFunction):
         return "Minimizes the number of alterations"
 
 
-
 class ModificationType(PhenotypeEvaluationFunction, KineticEvaluationFunction):
-    """This Objective function favors solutions with deletions, under expression and over expression, 
+    """This Objective function favors solutions with deletions, under expression and over expression,
     in this same order.
     """
 
-    def __init__(self, penalizations = {'KO':5,'UE':2, 'OE':0} ,maximize=True):
+    def __init__(self, penalizations={'KO': 5, 'UE': 2, 'OE': 0}, maximize=True):
         super(ModificationType, self).__init__(maximize=maximize, worst_fitness=0.0)
         self.penalizations = penalizations
-        
 
     def get_fitness(self, simulResult, candidate, **kwargs):
         sum = 0
         for v in candidate.values():
             if v == 0:
                 sum += self.penalizations['KO']
-            elif v < 1 :
+            elif v < 1:
                 sum += self.penalizations['UE']
             else:
                 sum += self.penalizations['OE']
-        return sum/len(candidate)            
+        return sum / len(candidate)
 
     def required_simulations(self):
         """
@@ -540,48 +547,3 @@ class ModificationType(PhenotypeEvaluationFunction, KineticEvaluationFunction):
 
     def method_str(self):
         return "ModificationType"
-
-
-
-
-
-"""
-Kinetic specific evaluation functions
-"""
-
-
-class KineticTargetFlux(KineticEvaluationFunction):
-    """ Target Flux evaluation function. 
-    The fitness value is the flux value of the identified reaction.
-
-    
-    :param reaction: (str) The reaction identifier whose flux value is to be used as fitness. Default None in which case the model objective is considered.
-    
-    """
-
-    def __init__(self, reaction, maximize = True):
-        super(KineticTargetFlux, self).__init__(maximize=maximize, worst_fitness=0.0)
-        self.reaction = reaction
-       
-    def get_fitness(self, simul_results, candidate, **kwargs):
-        if simul_results.status == ODEStatus.ERROR:
-            return self.worst_fitness
-        else:
-            try :
-                return simul_results.fluxes[self.reaction]
-            except Exception as e:
-                print(e)
-                return self.worst_fitness
-
-    def required_simulations(self):
-        """
-        If the evaluation function requires a pre-simulation to compute fitness values
-        """
-        return []
-
-    def short_str(self):
-        return "TargetFlux"
-
-    def method_str(self):
-        return "TargetFlux"
-
