@@ -1,11 +1,12 @@
-""" JMetal Problems 
+""" JMetal Problems
 """
-from jmetal.core.solution import Solution, IntegerSolution
-from jmetal.core.problem import Problem
-from mewpy.optimization.ea import SolutionInterface, dominance_test
-from typing import Tuple, List
 import random
-import warnings
+from typing import Tuple, List
+from jmetal.core.problem import Problem
+from jmetal.core.solution import Solution
+
+from ..ea import SolutionInterface, dominance_test
+from ...util.process import Evaluable
 
 # define EA representation for OU
 IntTupple = Tuple[int]
@@ -17,7 +18,7 @@ class KOSolution(Solution[int], SolutionInterface):
     def __init__(self, lower_bound: int, upper_bound: int, number_of_variables: int, number_of_objectives: int,
                  number_of_constraints: int = 0):
         super(KOSolution, self).__init__(number_of_variables,
-                                         number_of_objectives,number_of_constraints)
+                                         number_of_objectives, number_of_constraints)
         self.lower_bound = lower_bound
         self.upper_bound = upper_bound
 
@@ -64,7 +65,7 @@ class KOSolution(Solution[int], SolutionInterface):
 
     def get_representation(self):
         """
-        Returns a set representation of the candidate 
+        Returns a set representation of the candidate
         """
         return set(self.variables)
 
@@ -77,15 +78,14 @@ class KOSolution(Solution[int], SolutionInterface):
     def __str__(self):
         return " ".join((self.variables))
 
-    
-
 
 class OUSolution(Solution[IntTupple], SolutionInterface):
     """
     Class representing a Over/Under expression solution.
     """
 
-    def __init__(self, lower_bound: List[int], upper_bound: List[int], number_of_variables: int, number_of_objectives: int):
+    def __init__(self, lower_bound: List[int], upper_bound: List[int], number_of_variables: int,
+                 number_of_objectives: int):
         super(OUSolution, self).__init__(number_of_variables,
                                          number_of_objectives)
         self.upper_bound = upper_bound
@@ -142,9 +142,12 @@ class OUSolution(Solution[IntTupple], SolutionInterface):
         return " ".join((self.variables))
 
 
-class JMetalKOProblem(Problem[KOSolution]):
+class JMetalKOProblem(Problem[KOSolution], Evaluable):
 
-    def __init__(self, problem):
+    def __init__(self, problem, initial_polulation):
+        """JMetal OU problem. Encapsulates a MEWpy problem
+        so that it can be used in jMetal.
+        """
         self.problem = problem
         self.number_of_objectives = len(self.problem.fevaluation)
         self.obj_directions = []
@@ -155,9 +158,15 @@ class JMetalKOProblem(Problem[KOSolution]):
                 self.obj_directions.append(self.MAXIMIZE)
             else:
                 self.obj_directions.append(self.MINIMIZE)
+        self.initial_polulation = initial_polulation
+        self.__next_ini_sol = 0
 
     def create_solution(self) -> KOSolution:
-        solution = self.problem.generator(random, None)
+        if self.__next_ini_sol < len(self.initial_polulation):
+            solution = self.problem.encode(self.initial_polulation[self.__next_ini_sol])
+            self.__next_ini_sol += 1
+        else:
+            solution = self.problem.generator(random, None)
         new_solution = KOSolution(
             self.problem.bounder.lower_bound,
             self.problem.bounder.upper_bound,
@@ -166,6 +175,12 @@ class JMetalKOProblem(Problem[KOSolution]):
         new_solution.variables = list(solution)
         return new_solution
 
+    def reset_initial_population_counter(self):
+        """ Resets the pointer to the next initial population element.
+        This strategy is used to overcome the unavailable seeding API in jMetal.
+        """
+        self.__next_ini_sol = 0
+
     def get_constraints(self, solution):
         return self.problem.decode(set(solution.variables))
 
@@ -180,13 +195,22 @@ class JMetalKOProblem(Problem[KOSolution]):
                 solution.objectives[i] = p[i]
         return solution
 
+    def evaluator(self, candidates, *args):
+        res = []
+        for candidate in candidates:
+            res.append(self.evaluate(candidate))
+        return res
+
     def get_name(self) -> str:
         return self.problem.get_name()
 
 
-class JMetalOUProblem(Problem[OUSolution]):
+class JMetalOUProblem(Problem[OUSolution], Evaluable):
 
-    def __init__(self, problem):
+    def __init__(self, problem, initial_polulation=[]):
+        """JMetal OU problem. Encapsulates a MEWpy problem
+        so that it can be used in jMetal.
+        """
         self.problem = problem
         self.number_of_objectives = len(self.problem.fevaluation)
         self.obj_directions = []
@@ -197,11 +221,15 @@ class JMetalOUProblem(Problem[OUSolution]):
                 self.obj_directions.append(self.MAXIMIZE)
             else:
                 self.obj_directions.append(self.MINIMIZE)
+        self.initial_polulation = initial_polulation
+        self.__next_ini_sol = 0
 
     def create_solution(self) -> OUSolution:
-        # uses the super class generator
-        solution = self.problem.generator(random, None)
-
+        if self.__next_ini_sol < len(self.initial_polulation):
+            solution = self.problem.encode(self.initial_polulation[self.__next_ini_sol])
+            self.__next_ini_sol += 1
+        else:
+            solution = self.problem.generator(random, None)
         new_solution = OUSolution(
             self.problem.bounder.lower_bound,
             self.problem.bounder.upper_bound,
@@ -210,6 +238,9 @@ class JMetalOUProblem(Problem[OUSolution]):
         new_solution.variables = list(solution)
         return new_solution
 
+    def reset_initial_population_counter(self):
+        self.__next_ini_sol = 0
+
     def get_constraints(self, solution):
         return self.problem.decode(set(solution.variables))
 
@@ -224,5 +255,11 @@ class JMetalOUProblem(Problem[OUSolution]):
                 solution.objectives[i] = p[i]
         return solution
 
+    def evaluator(self, candidates, *args):
+        res = []
+        for candidate in candidates:
+            res.append(self.evaluate(candidate))
+        return res
+
     def get_name(self) -> str:
         return self.problem.get_name()
