@@ -1,7 +1,8 @@
-from .problem import AbstractKOProblem, AbstractOUProblem
-from mewpy.utils.constants import ModelConstants
-from collections import OrderedDict
 import warnings
+from collections import OrderedDict
+
+from .problem import AbstractKOProblem, AbstractOUProblem
+from ..util.constants import ModelConstants
 
 
 class GeckoKOProblem(AbstractKOProblem):
@@ -37,6 +38,7 @@ class GeckoKOProblem(AbstractKOProblem):
         #    raise Exception("The model should be an instance of GeckoModel")
         # problem simulation context
         self.prot_prefix = kwargs.get('prot_prefix', 'draw_prot_')
+        self.simulator.prot_prefix = self.prot_prefix
 
     def _build_target_list(self):
         """
@@ -115,7 +117,6 @@ class GeckoOUProblem(AbstractOUProblem):
             target = target - set(self.non_target)
         self._trg_list = list(target)
 
-
     def decode(self, candidate):
         """
         Decodes a candidate, an integer set, into a dictionary of constraints
@@ -126,13 +127,12 @@ class GeckoOUProblem(AbstractOUProblem):
 
                 decoded_candidate["{}{}".format(
                     self.prot_prefix, self.target_list[idx])] = self.levels[lv_idx]
-                
+
             except IndexError:
                 raise IndexError(
                     f"Index out of range: {idx} from {len(self.target_list[idx])}")
         return decoded_candidate
 
-
     def solution_to_constraints(self, candidate):
         """
         Converts a candidate, a set (protein,lv), into a dictionary of constraints
@@ -163,7 +163,7 @@ class GeckoOUProblem(AbstractOUProblem):
                 continue
             else:
                 constraints[rxn] = (
-                    lv*fluxe_wt, ModelConstants.REACTION_UPPER_BOUND)
+                    lv * fluxe_wt, ModelConstants.REACTION_UPPER_BOUND)
                 # Deals with reverse reactions associated with the protein.
                 # Strategy: The reaction direction with no flux in the wild type (reference) is KO.
                 if prot in self.prot_rev_reactions.keys():
