@@ -1,6 +1,7 @@
-from .problem import AbstractKOProblem, AbstractOUProblem
-from collections import OrderedDict
 import warnings
+from collections import OrderedDict
+
+from .problem import AbstractKOProblem, AbstractOUProblem
 
 
 class RKOProblem(AbstractKOProblem):
@@ -27,24 +28,26 @@ class RKOProblem(AbstractKOProblem):
             model, fevaluation=fevaluation, **kwargs)
 
     def _build_target_list(self):
+        """Default modification target builder.
+        Removes drains, transport and essential reactions
+        """
         reactions = set(self.simulator.reactions)
         essential = set(self.simulator.essential_reactions)
         drains = set(self.simulator.get_drains())
-        target = reactions - essential - drains
+        transport = set(self.simulator.get_transport_reactions())
+        target = reactions - essential - drains - transport
         if self.non_target is not None:
             target = target - set(self.non_target)
-        target =  list(target)
+        target = list(target)
         try:
-            from mewpy.utils.constants import EAConstants
+            from mewpy.util.constants import EAConstants
             if EAConstants.PROB_TARGET and self.product:
-                from mewpy.utils.graph import probabilistic_reaction_targets
-                target = probabilistic_reaction_targets(self.model,self.product,target)        
+                from mewpy.util.graph import probabilistic_reaction_targets
+                target = probabilistic_reaction_targets(self.model, self.product, target)
         except Exception as e:
             warnings.warn(str(e))
-            
-        self._trg_list = target
 
-   
+        self._trg_list = target
 
 
 class ROUProblem(AbstractOUProblem):
@@ -78,15 +81,15 @@ class ROUProblem(AbstractOUProblem):
         target = reactions  # - drains
         if self.non_target is not None:
             target = target - set(self.non_target)
-        target =  list(target)
+        target = list(target)
         try:
-            from mewpy.utils.constants import EAConstants
+            from ..util.constants import EAConstants
             if EAConstants.PROB_TARGET and self.product:
-                from mewpy.utils.graph import probabilistic_reaction_targets
-                target = probabilistic_reaction_targets(self.model,self.product,target)        
+                from ..util.graph import probabilistic_reaction_targets
+                target = probabilistic_reaction_targets(self.model, self.product, target)
         except Exception as e:
             warnings.warn(str(e))
-            
+
         self._trg_list = target
 
     def solution_to_constraints(self, candidate):
