@@ -1,5 +1,6 @@
 from abc import abstractclassmethod
 from collections import OrderedDict
+
 from mewpy.simulation import SimulationMethod
 
 
@@ -86,6 +87,27 @@ class Simulator(ModelContainer):
         """
         raise NotImplementedError
 
+    def __evaluator__(self, kwargs, candidate):
+        res = self.simulate(constraints=candidate, **kwargs)
+        return res
+
+    def simulate_mp(self, objective=None, method=SimulationMethod.FBA, maximize=True, constraints_list=None,
+                    reference=None,
+                    solver=None, n_mp=None, **kwargs):
+        from mewpy.util.process import get_fevaluator
+        args = {}
+        args['objective'] = objective
+        args['method'] = method
+        args['maximize'] = maximize
+        args['reference'] = reference
+        args.update(kwargs)
+        from functools import partial
+        func = partial(self.__evaluator__, args)
+
+        mp_evaluator = get_fevaluator(func)
+        res = mp_evaluator.evaluate(constraints_list, None)
+        return res
+
 
 class SimulationResult(object):
     """Class that represents simulation results and performs operations over them."""
@@ -115,6 +137,7 @@ class SimulationResult(object):
         # Constraints specific to the simulation
         self.simulation_constraints = simul_constraints if simul_constraints else OrderedDict()
 
+
     def get_constraints(self):
         """
         :returns: All constraints applied during the simulation both persistent and simulation specific.
@@ -131,6 +154,12 @@ class SimulationResult(object):
     def __str__(self):
         return "objective: {}\nStatus: {}".format(self.objective_value, self.status)
 
+    @property
+    def data_frame(self):
+        import pandas as pd
+        df = pd.DataFrame(list(self.fluxes.items()), columns=['Reaction ID', 'Flux'])
+        return df
+
     def get_net_conversion(self, biomassId=None):
         """Returns a string representation of the net conversion.
 
@@ -156,16 +185,16 @@ class SimulationResult(object):
                     if firstLeft:
                         firstLeft = False
                     else:
-                        left = left+" + "
-                    left = left+str(-1*fluxValue)
-                    left = left+" "+m[0]
+                        left = left + " + "
+                    left = left + str(-1 * fluxValue)
+                    left = left + " " + m[0]
                 else:
                     if firstRight:
                         firstRight = False
                     else:
-                        right = right+" + "
-                    right = right+str(fluxValue)
-                    right = right+" "+m[0]
+                        right = right + " + "
+                    right = right + str(fluxValue)
+                    right = right + " " + m[0]
 
         if biomassId and biomassId in ssFluxes.keys():
             biomassFlux = ssFluxes[biomassId]
@@ -173,7 +202,7 @@ class SimulationResult(object):
                 if firstRight:
                     firstRight = False
                 else:
-                    right = right+" + "
+                    right = right + " + "
                 right = right + str(biomassFlux)
                 right = right + " " + biomassId
 
